wApp.factory('CampaignIterator', function () {
    "use strict";

    var campaigns = {
        cur_pos: 0,
        list: [],
        fakeID: Date.now()
    };

    campaigns.setList = function(new_list){
        campaigns.list = new_list;
        campaigns.cur_pos = 0;
    };
    campaigns.nextItem = function(){
        var requested_pos = campaigns.cur_pos + 1;
        var item = campaigns.list[requested_pos];
        if (item) {
            campaigns.cur_pos++;
            return item;
        }
        return;
    };
    campaigns.prevItem = function(){
        var requested_pos = campaigns.cur_pos - 1;
        var item = campaigns.list[requested_pos];
        if (item) {
            campaigns.cur_pos--;
            return item;
        }
        return;
    };

    return campaigns;
});

wApp.controller('CampaignCtrl',
    ['$scope','$routeParams', '$sce', 'webService','CampaignIterator',
    function ($scope, $routeParams, $sce, webService, CampaignIterator) {

    "use strict";
    $scope.campaigns   = [];
    $scope.campaign    = {};

    $scope.count       = 0;
    $scope.types_list  = [];
    $scope.status_list = [];

    $scope.filter = {
        'type':         null,
        'status':       null,
        'filterString': ''
    };

    webService.get( 'campaign/globals').then(
        function(data){
            $scope.status_list = data.status;
            $scope.types_list =  data.types;
            $scope.count = data.count;
        },
        function(err){
            $log.error(['CampaignCtrl > no results',err]);
        }
    );

    $scope.n = 10;
    $scope.o = 0;

    $scope.pageStart = $scope.o + 1;
    $scope.pageEnd = $scope.o + $scope.campaigns.length;
    $scope.lastPage = false;

    $scope.trustSrc = function(src) {
        return $sce.trustAsResourceUrl( src );
    };

    $scope.acceptNext = function() {
        updateStatusAndNext($scope.campaign, 'approved');
    };

    $scope.noteNext = function(status) {
        $scope.campaign.status = status;
        $('.note-box').show();
        $('body').addClass('has-note-box');
    };

    $scope.nextCampaign = function() {
        gotoCampaign(CampaignIterator.nextItem());
    };

    $scope.prevCampaign = function() {
        gotoCampaign(CampaignIterator.prevItem());
    };

    $scope.nextPage = function () {
        if($scope.lastPage === false) {
            $scope.o += $scope.n;
            getCampaigns();
        }
    };

    $scope.$watch('status', function() {
        $scope.viewParams.filter = $scope.status;
    });

    $scope.prevPage = function () {
        if($scope.pageStart - $scope.n > 0) {
            $scope.o -= $scope.n;
            $scope.lastPage = false;
            getCampaigns();
        }
    };

    $scope.noteUpdate = function() {
        updateStatusAndNext($scope.campaign, $scope.campaign.status, $scope.campaign.note);
        $('.note-box').hide();
        $('body').removeClass('has-note-box');
    };
    $scope.noteCancle = function() {
        $scope.campaign.note='';
        $('.note-box').hide();
        $('body').removeClass('has-note-box');
    };

    $scope.refresh = function() {
        loadDetails( $scope.campaign.uuid );
    };

    var url = 'campaign',
        updatePagecount = function () {
            $scope.pageStart = $scope.o + 1;
            $scope.pageEnd = $scope.o + $scope.campaigns.length;
        },
        getCampaigns = function () {
            var queryObject = {
                "s": $scope.status,
                "n": $scope.n,
                "o": $scope.o
            };

            if($scope.lastPage === false) {
                webService.get( url, queryObject ).then(
                    function(data){
                        //Update UI using data or use the data to call another
                        if( ! data.isEmpty()){
                            if(data.length < $scope.n){
                                $scope.lastPage = true;
                            }
                            $scope.campaigns = data.map(function(item){
                                item.valid_from=Date.create(item.valid_from);
                                item.valid_until=Date.create(item.valid_until);
                                return item;
                            });

                            CampaignIterator.setList($scope.campaigns);
                            updatePagecount();
                        }
                    },
                    function(err){
                        console.error(['CampaingCtrl > no results',err]);
                    }

                );
            }

        },
        loadDetails = function ( id ) {
            var loadID = id || $routeParams.id;
            var durl = url + '/' + loadID + '?filter='+ $scope.viewParams.filter;
            webService.get( durl ).then(
                function(data){
                    $scope.campaign = data;
                    $scope.campaign.valid_from = Date.create(data.valid_from);
                    $scope.campaign.valid_until = Date.create(data.valid_until);
                },
                function(err){
                    console.error(['CampaignCtrl > no details',err]);
                }
            );
        },
        gotoCampaign = function(campaign) {
            if (campaign) {
                loadDetails( campaign.uuid );
            }
            else {
                alert("NO MORE CAMPAIGNS");
            }
        },
        updateStatusAndNext = function(campaign, status, note) {
            webService.put(
                'campaign/' + campaign.uuid + '/status',
                { status: status,
                    note: note
                }
            ).then(
                function(data){
                    $scope.campaign.status = status;
                    $scope.campaign.note = '';
                    gotoCampaign(CampaignIterator.nextItem());
                },
                function(err){
                    console.error(['Could not set status to approved',err]);
                }
            );
        };

    if($routeParams.id) {
        loadDetails();
    }
    else {
        getCampaigns();
    }

}]);
