package Winxle::Admin;
use 5.018;
use OX;
use Plack::App::File;
use Plack::Runner;
use Path::Class qw();

our $VERSION = 0.001;

use OX::RouteBuilder::REST;

with 'Winxle::Role::Ox::Plack' => { port_offset => 100 };

sub request_class {'Winxle::Request'}

has 'root_dir' => (
    is        => 'ro',
    lifecycle => 'Singleton',
    isa       => 'Path::Class::Dir',
    required  => 1,
);

has 'static_dir' => (
    is        => 'ro',
    lifecycle => 'Singleton',
    isa       => 'Path::Class::Dir',
    required  => 1,
);

has 'js_env_overwrite' => (
    is        => 'ro',
    lifecycle => 'Singleton',
    default=>'root/static/assets/js/environment_server_running.js',
);

has 'merchant_model' => (is=>'ro',isa=>'Winxle::Admin::Model::Merchant',required=>1,lifecycle=>'Singleton');
has 'campaign_model' => (is=>'ro',isa=>'Winxle::Admin::Model::Campaign',required=>1,lifecycle=>'Singleton');
has 'user_model' => (is=>'ro',isa=>'Winxle::Admin::Model::User',required=>1,lifecycle=>'Singleton');
has 'l10n_model' => (is=>'ro',isa=>'Winxle::Admin::Model::L10n',required=>1,lifecycle=>'Singleton');

has 'merchant_ctrl' => (
    is    => 'ro',
    isa   => 'Winxle::Admin::Controller::Merchant',
    infer => 1,
);

has 'user_ctrl' => (
    is    => 'ro',
    isa   => 'Winxle::Admin::Controller::User',
    infer => 1,
);

has 'l10n_ctrl' => (
    is    => 'ro',
    isa   => 'Winxle::Admin::Controller::L10n',
    infer => 1,
);
has 'campaign_ctrl' => (
    is    => 'ro',
    isa   => 'Winxle::Admin::Controller::Campaign',
    infer => 1,
);

use Winxle::Util::Path;

router as {

    wrap 'Winxle::Admin::StupidPlackAuth';

    mount '/static' => "Plack::App::File", ( root => 'static_dir', );

    # this is a bit ugly, but still nicer than setting up a controller
    route '/' => sub {
        my $r = $_[0]->new_response;
        $r->content(
            scalar Path::Class::file( Winxle::Util::Path::home_from_package(),
                qw(root static index.html) )->slurp
        );
        $r->status(200);
        return $r->finalize;
    };

    route '/campaign' => 'REST.campaign_ctrl.root';
    route '/campaign/globals' => 'campaign_ctrl.globals';
    route '/campaign/:uuid' => 'REST.campaign_ctrl.item';
    route '/campaign/:uuid/status' => 'REST.campaign_ctrl.status';

    route '/merchant' => 'REST.merchant_ctrl.root';
    route '/merchant/globals' => 'merchant_ctrl.globals';
    route '/merchant/:uuid' => 'REST.merchant_ctrl.item';

    route '/user' => 'REST.user_ctrl.root';
    route '/user/globals' => 'user_ctrl.globals';
    route '/user/:id' => 'REST.user_ctrl.item';

    route '/l10n/available_langs' => 'REST.l10n_ctrl.available_langs';
    route '/l10n/:lang' => 'REST.l10n_ctrl.root';
    route '/l10n/update' => 'REST.l10n_ctrl.update';

    mount '/static/assets/js/environment.js' => "Plack::App::File",
        ( file => 'js_env_overwrite' );

};

1;

