package Winxle::Request;

use 5.014;
use Moose;
use JSON::XS qw();
use namespace::autoclean;

extends 'OX::Request';

has stash => ( is => 'ro', default => sub { {} } );

has 'is_xhr' => ( is => 'ro', isa => 'Bool', lazy_build => 1 );

sub _build_is_xhr {
    my $self     = shift;
    my $req_with = $self->headers->header('X-Requested-With');
    return 1 if ( $req_with && lc($req_with) eq 'xmlhttprequest' );
    return;
}

has 'base_url' => ( is => 'ro', isa => 'Str', lazy_build => 1 );

sub _build_base_url {
    my $self = shift;
    my $uri  = $self->uri;
    my $port = $uri->port;
    if ( !$port || $port == 80 || $port == 443 ) {
        return $uri->scheme . '://' . $uri->host . '/';
    }
    else {
        return $uri->scheme . '://' . $uri->host_port . '/';
    }
}

sub absolute_url {
    my ( $self, $url ) = @_;
    $url =~ s{^/}{};
    return $self->base_url . $url;
}

sub _json_encode {
    my ( $self, @rest ) = @_;
    my $json = JSON::XS->new->allow_nonref->pretty(1);
    return $json->encode(@rest);
}

sub got_json {
    my $self = shift;
    my $got = $self->content;
    return JSON::XS::decode_json($got);
}

sub new_json_response {
    my ( $self, $data ) = @_;

    return $self->new_response(
        headers => [ content_type => 'application/json' ],
        status  => 200,
        content => $self->_json_encode($data),
    );
}

sub new_json_error {
    my ( $self, $message, $status ) = @_;
    $status||=400;
    return $self->new_response(
        headers => [ content_type => 'application/json' ],
        status  => $status,
        content => $self->_json_encode(
            { status => 'error', message => "$message" }
        ),
    );
}

sub redirect {
    my ( $self, $target ) = @_;
    my $res = $self->new_response();
    if ( ref($target) eq 'HASH' ) {
        $target = $self->uri_for($target);
    }
    $res->redirect($target);
    return $res->finalize;
}

__PACKAGE__->meta->make_immutable;

1;
